@@ -135,11 +135,10 @@ from __future__ import with_statement
 
 __ver_major__ = 0
 __ver_minor__ = 7
-__ver_patch__ = 1
+__ver_patch__ = 2
 __ver_sub__ = ""
 __version__ = "%d.%d.%d%s" % (__ver_major__,__ver_minor__,__ver_patch__,__ver_sub__)
 
-
 import os
 import sys
 import shutil
@@ -259,6 +258,7 @@ class Esky(object):
         details = split_app_version(best_version)
         self.name,self.version,self.platform = details
 
+    @allow_from_sudo()
     def lock(self,num_retries=0):
         """Lock the application directory for exclusive write access.
 
@@ -273,6 +273,8 @@ class Esky(object):
         This also has the side-effect of failing early if the user does not
         have permission to modify the application directory.
         """
+        if self.sudo_proxy is not None:
+            return self.sudo_proxy.lock()
         if num_retries > 5:
             raise EskyLockedError
         if threading:
@@ -319,8 +321,11 @@ class Esky(object):
             self._lock_count = 1
             return True
             
+    @allow_from_sudo()
     def unlock(self):
         """Unlock the application directory for exclusive write access."""
+        if self.sudo_proxy is not None:
+            return self.sudo_proxy.unlock()
         self._lock_count -= 1
         if self._lock_count == 0:
             if threading:
@@ -375,77 +380,131 @@ class Esky(object):
         """
         if self.sudo_proxy is not None:
             return self.sudo_proxy.cleanup()
-        appdir = self.appdir
+        if not self.needs_cleanup():
+            return True
         self.lock()
-        success = True
         try:
-            best_version = get_best_version(appdir)
-            new_version = get_best_version(appdir,include_partial_installs=True)
-            #  If there's a partial install we must complete it, since it
-            #  could have left exes in the bootstrap env and we don't want
-            #  to accidentally delete their dependencies.
-            if best_version != new_version:
-                (_,v,_) = split_app_version(new_version)
-                self.install_version(v)
-                best_version = new_version
-            #  Now we can safely remove all the old versions.
-            #  We except the currently-executing version, and silently
-            #  ignore any locked versions.
-            manifest = self._version_manifest(best_version)
-            manifest.add("updates")
-            manifest.add("locked")
-            manifest.add(best_version)
-            if self.active_version and self.active_version != best_version:
-                success = False
-                manifest.add(self.active_version)
-            for nm in os.listdir(appdir):
-                if nm not in manifest:
-                    fullnm = os.path.join(appdir,nm)
-                    if is_version_dir(fullnm):
-                        #  It's an installed-but-obsolete version.  Properly
-                        #  uninstall it so it will clean up the bootstrap env.
-                        (_,v,_) = split_app_version(nm)
-                        try:
-                            self.uninstall_version(v)
-                        except VersionLockedError:
-                            success = False
+            #  This is a little coroutine trampoline that executes each
+            #  action yielded from self._cleanup_actions().  Any exceptions
+            #  that the action raises are thrown back into the generator.
+            #  The result of each is and-ed into the success code.
+            #
+            #  If you're looking for the actual logic of the cleanup process,
+            #  it's all in the _cleanup_actions() method.
+            success = True
+            actions = self._cleanup_actions()
+            try:
+                act = lambda: True
+                while True:
+                    try:
+                        if callable(act):
+                            res = act()
+                        elif len(act) == 1:
+                            res = act[0]()
+                        elif len(act) == 2:
+                            res = act[0](*act[1])
                         else:
-                            success &= self._try_remove(appdir,nm,manifest)  
-                    elif is_uninstalled_version_dir(fullnm):
-                        #  It's a partially-removed version; finish removing it.
-                        success &= not self._try_remove(appdir,nm,manifest)
-                    elif ".old." in nm or nm.endswith(".old"):
-                        #  It's a temporary backup file; remove it.
-                        success &= not self._try_remove(appdir,nm,manifest)
+                            res = act[0](*act[1],**act[2])
+                        if res is not None:
+                            success &= res
+                    except Exception:
+                        act = actions.throw(*sys.exc_info())
                     else:
-                        #  It's an unaccounted-for entry in the bootstrap env.
-                        #  Can't prove it's safe to remove, so leave it.
-                        pass
-            #  If there are pending overwrites, try to do them.
-            ovrdir = os.path.join(appdir,best_version,"esky-overwrite")
-            if os.path.exists(ovrdir):
-                try:
-                    for (dirnm,_,filenms) in os.walk(ovrdir,topdown=False):
-                        for nm in filenms:
-                            ovrsrc = os.path.join(dirnm,nm)
-                            ovrdst = os.path.join(appdir,ovrsrc[len(ovrdir)+1:])
-                            with open(ovrsrc,"rb") as fIn:
-                                with open(ovrdst,"ab") as fOut:
-                                    fOut.seek(0)
-                                    chunk = fIn.read(512*16)
-                                    while chunk:
-                                        fOut.write(chunk)
-                                        chunk = fIn.read(512*16)
-                            os.unlink(ovrsrc)
-                        os.rmdir(dirnm)
-                except EnvironmentError, e:
-                    success = False
-            #  Get the VersionFinder to clean up after itself
-            if self.version_finder is not None:
-                self.version_finder.cleanup(self)
+                        act = actions.next()
+            except StopIteration:
+                return success
         finally:
             self.unlock()
-        return success
+
+    def needs_cleanup(self):
+        """Check whether a call to cleanup() is necessary.
+
+        This method checks whether a call to the cleanup() method will have
+        any work to do, without obtaining a lock on the esky's appdir.  You
+        might like to use this to avoid locking the appdir (which may require
+        elevating to root) when there's nothing to do.
+        """
+        for act in self._cleanup_actions():
+            return True
+        return False
+
+    def _cleanup_actions(self):
+        """Iterator giving (func,args,kwds) tuples of cleanup actions.
+
+        This encapsulates the logic of the "cleanup" method without actually
+        performing any of the actions, making it easy to check whether cleanup
+        is required without duplicating the logic.
+        """
+        appdir = self.appdir
+        best_version = get_best_version(appdir)
+        new_version = get_best_version(appdir,include_partial_installs=True)
+        #  If there's a partial install we must complete it, since it
+        #  could have left exes in the bootstrap env and we don't want
+        #  to accidentally delete their dependencies.
+        if best_version != new_version:
+            (_,v,_) = split_app_version(new_version)
+            yield (self.install_version,(v,))
+            best_version = new_version
+        #  Now we can safely remove all the old versions.
+        #  We except the currently-executing version, and silently
+        #  ignore any locked versions.
+        manifest = self._version_manifest(best_version)
+        manifest.add("updates")
+        manifest.add("locked")
+        manifest.add(best_version)
+        if self.active_version and self.active_version != best_version:
+            yield lambda: False
+            manifest.add(self.active_version)
+        for nm in os.listdir(appdir):
+            if nm not in manifest:
+                fullnm = os.path.join(appdir,nm)
+                if is_version_dir(fullnm):
+                    #  It's an installed-but-obsolete version.  Properly
+                    #  uninstall it so it will clean up the bootstrap env.
+                    (_,v,_) = split_app_version(nm)
+                    try:
+                        yield (self.uninstall_version, (v,))
+                    except VersionLockedError:
+                        yield lambda: False
+                    else:
+                        yield (self._try_remove, (appdir,nm,manifest,))
+                elif is_uninstalled_version_dir(fullnm):
+                    #  It's a partially-removed version; finish removing it.
+                    yield (self._try_remove, (appdir,nm,manifest,))
+                elif ".old." in nm or nm.endswith(".old"):
+                    #  It's a temporary backup file; remove it.
+                    yield (self._try_remove, (appdir,nm,manifest,))
+                else:
+                    #  It's an unaccounted-for entry in the bootstrap env.
+                    #  Can't prove it's safe to remove, so leave it.
+                    pass
+        #  If there are pending overwrites, try to do them.
+        ovrdir = os.path.join(appdir,best_version,"esky-overwrite")
+        if os.path.exists(ovrdir):
+            try:
+                for (dirnm,_,filenms) in os.walk(ovrdir,topdown=False):
+                    for nm in filenms:
+                        ovrsrc = os.path.join(dirnm,nm)
+                        ovrdst = os.path.join(appdir,ovrsrc[len(ovrdir)+1:])
+                        yield (self._overwrite, (ovrsrc,ovrdst,))
+                        yield (os.unlink, (ovrsrc,))
+                    yield (os.rmdir, (dirnm,))
+            except EnvironmentError:
+                yield lambda: False
+        #  Get the VersionFinder to clean up after itself
+        if self.version_finder is not None:
+            if self.version_finder.needs_cleanup(self):
+                yield (self.version_finder.cleanup, (self,))
+
+    def _overwrite(self,src,dst):
+        """Directly overwrite file 'dst' with the contents of file 'src'."""
+        with open(src,"rb") as fIn:
+            with open(dst,"ab") as fOut:
+                fOut.seek(0)
+                chunk = fIn.read(512*16)
+                while chunk:
+                   fOut.write(chunk)
+                   chunk = fIn.read(512*16)
 
     @allow_from_sudo()
     def cleanup_at_exit(self):
@@ -463,19 +522,35 @@ class Esky(object):
             return self.sudo_proxy.cleanup_at_exit()
         if not getattr(sys,"frozen",False):
             exe = [sys.executable,"-c","import esky; esky.run_startup_hooks()","--esky-spawn-cleanup"]
-        elif os.path.basename(sys.executable).lower() in ("python","pythonw"):
-            exe = [sys.executable,"-c","import esky; esky.run_startup_hooks()","--esky-spawn-cleanup"]
         else:
-            if not _startup_hooks_were_run:
-                raise OSError(None,"unable to sudo: startup hooks not run")
-            exe = [sys.executable,"--esky-spawn-cleanup"]
+            exe = sys.executable
+            #  Try to re-launch the best available version, so that the
+            #  currently in-use version can be cleaned up.
+            if self.active_version is not None:
+               appdir = self.appdir
+               bestver = get_best_version(appdir,include_partial_installs=True)
+               (_,version,_) = split_app_version(bestver)
+               if self.active_version != version:
+                   if self.active_version in exe:
+                       exe = exe.replace(self.active_version,version)
+                       if not os.path.isfile(exe):
+                           exe = sys.executable
+            if os.path.basename(exe).lower() in ("python","pythonw"):
+                exe = [exe,"-c","import esky; esky.run_startup_hooks()","--esky-spawn-cleanup"]
+            else:
+                if not _startup_hooks_were_run:
+                    raise OSError(None,"unable to cleanup: startup hooks not run")
+                exe = [exe,"--esky-spawn-cleanup"]
         exe = exe + [b64encode(pickle.dumps(self,pickle.HIGHEST_PROTOCOL))]
         @atexit.register
         def spawn_cleanup():
             rnul = open(os.devnull,"r")
             wnul = open(os.devnull,"w")
             if sys.platform == "win32":
-                kwds = dict(close_fds=True)
+                if sys.hexversion >= 0x20600000:
+                    kwds = dict(close_fds=True)
+                else:
+                    kwds = {}
             else:
                 kwds = dict(stdin=rnul,stdout=wnul,stderr=wnul,close_fds=True)
             subprocess.Popen(exe,**kwds)
